@@ -2,7 +2,9 @@
  *                            COPYRIGHT
  *
  *  cschem - modular/flexible schematics editor - libcschem (core library)
- *  Copyright (C) 2019 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2019, 2024 Tibor 'Igor2' Palinkas
+ *
+ *  (Supported by NLnet NGI0 Entrust in 2023)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -30,8 +32,12 @@
 #include <genht/htsp.h>
 #include <genht/hash.h>
 
-#include "abstract.h"
 #include <librnd/core/compat_misc.h>
+#include <librnd/core/error.h>
+
+#include "hierarchy.h"
+
+#include "abstract.h"
 
 static const char *atype_names[] = {
 	"<invalid>",
@@ -61,6 +67,7 @@ void csch_anet_free_fields(csch_anet_t *net)
 {
 	vtp0_uninit(&net->conns);
 	free(net->name);
+	free(net->name_loc);
 	csch_ahdr_uninit(&net->hdr);
 }
 
@@ -86,6 +93,7 @@ void csch_acomp_free_fields(csch_acomp_t *comp)
 	htsp_uninit(&comp->ports);
 	htsp_uninit(&comp->busports);
 	free(comp->name);
+	free(comp->name_loc);
 	csch_ahdr_uninit(&comp->hdr);
 }
 
@@ -96,12 +104,16 @@ void csch_abstract_init(csch_abstract_t *abs)
 	htsp_init(&abs->nets, strhash, strkeyeq);
 	htsp_init(&abs->comps, strhash, strkeyeq);
 	htip_init(&abs->aid2obj, longhash, longkeyeq);
+	abs->hroot = csch_hlevel_new(NULL, NULL, NULL);
 	abs->next_aid = 1;
 }
 
 void csch_abstract_uninit(csch_abstract_t *abs)
 {
 	htsp_entry_t *e;
+	long n;
+
+	csch_hlevel_free_tree(abs->hroot);
 
 	for(e = htsp_first(&abs->nets); e != NULL; e = htsp_next(&abs->nets, e)) {
 		csch_anet_free_fields(e->value);
@@ -113,9 +125,16 @@ void csch_abstract_uninit(csch_abstract_t *abs)
 		free(e->value); e->value = NULL;
 	}
 
+	for(n = 0; n < abs->orphaned_ports.used; n++) {
+		csch_aport_free_fields(abs->orphaned_ports.array[n]);
+		free(abs->orphaned_ports.array[n]);
+	}
+
 	htsp_uninit(&abs->nets);
 	htsp_uninit(&abs->comps);
 	htip_uninit(&abs->aid2obj);
+	vtp0_uninit(&abs->orphaned_ports);
+
 	memset(abs, 0, sizeof(csch_abstract_t));
 }
 
@@ -149,40 +168,40 @@ RND_INLINE void csch_aobj_init(csch_abstract_t *abs, csch_ahdr_t *dst, csch_atyp
 	}
 }
 
-csch_anet_t *csch_anet_get(csch_abstract_t *abs, const char *netname, int alloc, int set_no_uname)
+csch_ascope_t csch_split_name_scope(const char **name)
 {
-	csch_anet_t *net;
+	const char *nm = *name;
 
-	net = htsp_get(&abs->nets, netname);
-	if ((net != NULL) || (!alloc))
-		return net;
+	if (nm[0] == '/') {
+		(*name) += 1;
+		return CSCH_ASCOPE_GLOBAL;
+	}
 
-	net = calloc(sizeof(csch_anet_t), 1);
-	csch_aobj_init(abs, &net->hdr, CSCH_ATYPE_NET);
-	net->name = rnd_strdup(netname);
-	net->no_uname = set_no_uname;
-	htsp_set(&abs->nets, net->name, net);
-	return net;
+	/* check this so nm[1] is safe to address */
+	if (nm[0] == '\0')
+		return CSCH_ASCOPE_AUTO;
+
+	if (nm[1] == '/') {
+		if (nm[0] == '.') {
+			(*name) += 2;
+			return CSCH_ASCOPE_SHEET_LOCAL;
+		}
+		if (nm[0] == '^') {
+			(*name) += 2;
+			return CSCH_ASCOPE_SUBTREE_AUTO;
+		}
+		if (nm[0] == 'v') {
+			(*name) += 2;
+			return CSCH_ASCOPE_SUBTREE_LOCAL;
+		}
+	}
+
+	/* no usable prefix */
+	return CSCH_ASCOPE_AUTO;
 }
 
-csch_acomp_t *csch_acomp_get(csch_abstract_t *abs, const char *name, int alloc)
-{
-	csch_acomp_t *comp;
-
-	if (name == NULL)
-		return NULL;
-
-	comp = htsp_get(&abs->comps, name);
-	if ((comp != NULL) || (!alloc))
-		return comp;
-
-	comp = calloc(sizeof(csch_acomp_t), 1);
-	csch_aobj_init(abs, &comp->hdr, CSCH_ATYPE_COMP);
-	comp->name = rnd_strdup(name);
-	htsp_set(&abs->comps, comp->name, comp);
-	htsp_init(&comp->ports, strhash, strkeyeq);
-	return comp;
-}
+#include "abs_net.c"
+#include "abs_comp.c"
 
 csch_aport_t *csch_aport_get(csch_abstract_t *abs, csch_acomp_t *comp, const char *name, int alloc)
 {
@@ -193,9 +212,6 @@ csch_aport_t *csch_aport_get(csch_abstract_t *abs, csch_acomp_t *comp, const cha
 		if ((port != NULL) || (!alloc))
 			return port;
 	}
-	else {
-		TODO("hierarchic: we probably need a per sheet hash of terminals");
-	}
 
 	port = calloc(sizeof(csch_aport_t), 1);
 	csch_aobj_init(abs, &port->hdr, CSCH_ATYPE_PORT);
@@ -205,7 +221,7 @@ csch_aport_t *csch_aport_get(csch_abstract_t *abs, csch_acomp_t *comp, const cha
 	if (comp != NULL)
 		htsp_set(&comp->ports, port->name, port);
 	else
-		TODO("hierarchic: we probably need a per sheet hash of terminals");
+		vtp0_append(&abs->orphaned_ports, port);
 	return port;
 }
 
