@@ -322,12 +322,12 @@ void csch_attrib_set_export_name(csch_attrib_t *dst, const char *expname)
 	dst->export_name = rnd_strdup(expname);
 }
 
-csch_attrib_t *csch_attrib_dup(csch_attrib_t *src)
+csch_attrib_t *csch_attrib_dup_rename(csch_attrib_t *src, const char *new_key)
 {
 	size_t n;
 	csch_attrib_t *dst = calloc(sizeof(csch_attrib_t), 1);
 
-	dst->key = rnd_strdup(src->key);
+	dst->key = rnd_strdup(new_key);
 	if (src->val != NULL)
 		dst->val = rnd_strdup(src->val);
 	else
@@ -350,6 +350,12 @@ csch_attrib_t *csch_attrib_dup(csch_attrib_t *src)
 	return dst;
 }
 
+csch_attrib_t *csch_attrib_dup(csch_attrib_t *src)
+{
+	return csch_attrib_dup_rename(src, src->key);
+}
+
+
 int csch_attrib_del(csch_attribs_t *attribs, int prio, const char *key, csch_source_arg_t *source)
 {
 	csch_attrib_t *a = htsp_get((csch_attribs_t *)attribs, key);
@@ -411,7 +417,11 @@ void csch_attrib_copy_all_undoable(csch_sheet_t *sheet, csch_cgrp_t *dst_obj, cs
 		csch_attrib_t *s = csch_attrib_dup(e->value);
 		csch_attrib_t *d = csch_attrib_get(dst, e->key);
 
-		if ((s != NULL) && (d != NULL) && (csch_attrib_val_eq_(s, d))) continue; /* do not merge if values are the same */
+		if ((s != NULL) && (d != NULL) && (csch_attrib_val_eq_(s, d))) {
+			/* do not merge if values are the same */
+			attr_free(s);
+			continue;
+		}
 
 		if (s->val == NULL) { /* source is an array */
 			int done = 0;
@@ -453,6 +463,8 @@ TODO("handle if dst is an array");
 					csch_attr_modify_str(sheet, dst_obj, s->prio, e->key, s->val, source, 1);
 				else
 					csch_attrib_set(dst, s->prio, e->key, s->val, source, NULL);
+				attr_free(s);
+				s = NULL;
 			}
 		}
 	}
