@@ -261,7 +261,7 @@ void csch_chdr_pen_name_modify(csch_sheet_t *sheet, csch_chdr_t *obj, csch_comm_
 }
 
 
-csch_inline void select_side_effects(csch_sheet_t *sheet, csch_chdr_t *obj, int root)
+RND_INLINE void select_side_effects(csch_sheet_t *sheet, csch_chdr_t *obj, int root)
 {
 	if (obj->type == CSCH_CTYPE_TEXT)
 		csch_text_invalidate_font((csch_text_t *)obj); /* may need to re-render for indicate selection */
