@@ -246,10 +246,20 @@ void csch_cgrp_bbox_update(csch_sheet_t *sheet, csch_cgrp_t *grp)
 		if (csch_bbox_is_invalid(&h->bbox))
 			continue;
 
-		csch_bbox_bump(&grp->bbox_flt, x, h->bbox.x1);
-		csch_bbox_bump(&grp->bbox_flt, x, h->bbox.x2);
-		csch_bbox_bump(&grp->bbox_flt, y, h->bbox.y1);
-		csch_bbox_bump(&grp->bbox_flt, y, h->bbox.y2);
+		if (csch_obj_is_grp(h)) { /* group child: should use the child's bbox that includes floaters */
+			csch_cgrp_t *hg = (csch_cgrp_t *)h;
+			assert(hg->bbox_flt.x1 != CSCH_COORD_INV); /* floater bbox of child can not be invalid if the above bbox validity check passed because normal bbx and flt bbox are updated in parallel */
+			csch_bbox_bump(&grp->bbox_flt, x, hg->bbox_flt.x1);
+			csch_bbox_bump(&grp->bbox_flt, x, hg->bbox_flt.x2);
+			csch_bbox_bump(&grp->bbox_flt, y, hg->bbox_flt.y1);
+			csch_bbox_bump(&grp->bbox_flt, y, hg->bbox_flt.y2);
+		}
+		else {
+			csch_bbox_bump(&grp->bbox_flt, x, h->bbox.x1);
+			csch_bbox_bump(&grp->bbox_flt, x, h->bbox.x2);
+			csch_bbox_bump(&grp->bbox_flt, y, h->bbox.y1);
+			csch_bbox_bump(&grp->bbox_flt, y, h->bbox.y2);
+		}
 
 		if (!h->floater) {
 			csch_bbox_bump(&grp->hdr.bbox, x, h->bbox.x1);
@@ -485,9 +495,6 @@ static void cgrp_rotate(csch_sheet_t *sheet, csch_chdr_t *obj, csch_coord_t rcx,
 	csch_coord_t nx = cgrp->x, ny = cgrp->y;
 	double nrot, cs, sn, rad;
 
-	if (cgrp->xform.mirx ^ cgrp->xform.miry)
-		da = -da;
-
 	nrot = cgrp->spec_rot + da * cgrp_rotdir(cgrp);
 	rad = -da / RND_RAD_TO_DEG;
 	cs = cos(rad); sn = sin(rad);
@@ -503,9 +510,6 @@ static void cgrp_rotate90(csch_sheet_t *sheet, csch_chdr_t *obj, csch_coord_t rc
 	csch_coord_t nx = cgrp->x, ny = cgrp->y;
 	double nrot;
 
-	if (cgrp->xform.mirx ^ cgrp->xform.miry)
-		n = -n;
-
 	nrot = fmod(cgrp->spec_rot + (double)n * 90.0 * cgrp_rotdir(cgrp), 360.0);
 	csch_rotate90_pt(&nx, &ny, rcx, rcy, n);
 
