@@ -181,11 +181,13 @@ static void poly_remove_undo(csch_undo_remove_t *slot)
 	csch_cnc_common_remove_undo(slot, CSCH_REM_FROM_RTREE | CSCH_REM_FROM_PARENT | CSCH_REM_DEL_EMPTY_PARENT);
 }
 
-static int poly_isc_with_box(csch_chdr_t *obj, csch_rtree_box_t *box)
+/* obj_idx is at most 2 items long but can be NULL to skip the search */
+RND_INLINE int poly_isc_with_box_(csch_chdr_t *obj, csch_rtree_box_t *box, long *obj_idx)
 {
 	csch_cpoly_t *poly = (csch_cpoly_t *)obj;
 	g2d_box_t gbox, pbox, obox;
 	long n;
+	int found = 0;
 	csch_coutline_t *o;
 
 	gbox.p1.x = box->x1; gbox.p1.y = box->y1;
@@ -206,10 +208,88 @@ static int poly_isc_with_box(csch_chdr_t *obj, csch_rtree_box_t *box)
 		if (!g2d_isc_box_box(&gbox, &obox))
 			continue;
 
-		if (csch_isc_with_box(&o->hdr, box))
-			return 1;
+		if (csch_isc_with_box(&o->hdr, box)) {
+			if (obj_idx != NULL) {
+				obj_idx[found] = n;
+				found++;
+				if (found == 2)
+					return 2;
+			}
+			else
+				return 1;
+		}
 	}
 
+	return found;
+}
+
+static int poly_isc_with_box(csch_chdr_t *obj, csch_rtree_box_t *box)
+{
+	return poly_isc_with_box_(obj, box, NULL);
+}
+
+int csch_poly_contour_objs_at(csch_cpoly_t *poly, csch_coord_t x, csch_coord_t y, csch_coord_t r, long out[2])
+{
+	csch_rtree_box_t box;
+	long obj_idx[2];
+	int len;
+
+	box.x1 = x - r; box.y1 = y - r;
+	box.x2 = x + r; box.y2 = y + r;
+	len = poly_isc_with_box_(&poly->hdr, &box, obj_idx);
+	switch(len) {
+		case 0:
+			out[0] = out[1] = -1;
+			return 0;
+		case 1:
+			out[0] = obj_idx[0];
+			out[1] = -1;
+			return 1;
+		case 2:
+			/* need to sort objects */
+			if (obj_idx[1] == obj_idx[0]+1) {
+				out[0] = obj_idx[0];
+				out[1] = obj_idx[1];
+				return 2;
+			}
+			else if ((obj_idx[0] == 0) && (obj_idx[1] == poly->outline.used - 1)) {
+				out[0] = obj_idx[1];
+				out[1] = obj_idx[0];
+				return 2;
+			}
+			else { /* self-isect poly? */
+				out[0] = obj_idx[0];
+				out[1] = -1;
+				return 1;
+			}
+			break;
+	}
+
+	assert(!"internal error poly_isc_with_box_()");
+	return 0;
+}
+
+int csch_poly_contour_obj_ends_at(csch_cpoly_t *poly, long idx, csch_coord_t x, csch_coord_t y)
+{
+	csch_coutline_t *o;
+	csch_coord_t ex, ey;
+
+	if ((idx < 0) || (idx >= poly->outline.used))
+		return 0;
+
+	o = &poly->outline.array[idx];
+
+	switch(o->hdr.type) {
+		case CSCH_CTYPE_LINE:
+			if ((o->line.inst.c.p1.x == x) && (o->line.inst.c.p1.y == y)) return 1;
+			if ((o->line.inst.c.p2.x == x) && (o->line.inst.c.p2.y == y)) return 1;
+			return 0;
+		case CSCH_CTYPE_ARC:
+			if ((csch_arc_get_endxy(&o->arc, 0, &ex, &ey) == 0) && (ex == x) && (ey == y)) return 1;
+			if ((csch_arc_get_endxy(&o->arc, 1, &ex, &ey) == 0) && (ex == x) && (ey == y)) return 1;
+			return 0;
+		default: break; /* Invalid object in polygon, already warned on create/load */
+	}
 	return 0;
 }
 
@@ -459,8 +539,17 @@ static int undo_poly_modify_swap(void *udata)
 	csch_cobj_redraw(u->poly);
 
 	if (u->swap_has) {
+		csch_sheet_t *sheet = u->poly->hdr.sheet;
+
+		/* if has_fill changes the fill layer rtree needs to be updated accordingly */
+		if (u->poly->has_fill && !u->has_fill)
+			csch_rtree_delete(&sheet->dsply_fill[u->poly->hdr.dsply], &u->poly->hdr, &u->poly->hdr.bbox);
+		else if (!u->poly->has_fill && u->has_fill)
+			csch_rtree_insert(&sheet->dsply_fill[u->poly->hdr.dsply], &u->poly->hdr, &u->poly->hdr.bbox);
+
 		rnd_swap(int, u->has_stroke, u->poly->has_stroke);
 		rnd_swap(int, u->has_fill, u->poly->has_fill);
+
 	}
 	if (u->swap_outline) {
 		rnd_swap(csch_vtcoutline_t, u->outline, u->poly->outline);
