@@ -36,13 +36,18 @@
 #include "buildin.h"
 #include "project_act.h"
 #include "project.h"
+#include "project_p4.h"
 #include "plug_library.h"
+#include "plug_io.h"
 #include "integrity.h"
 #include "undo.h"
 #include "actions_csch.h"
 #include "drc.h"
 
 const int *csch_cfg_multiport_net_merge = NULL;
+const int *csch_cfg_hier_net_allow_rename = NULL;
+const int *csch_cfg_hier_net_rename_prefer_top = NULL;
+
 minuid_session_t csch_minuid;
 
 
@@ -66,6 +71,12 @@ void csch_uninit(void)
 	csch_plug_library_uninit();
 }
 
+void csch_uninit_last(void)
+{
+	csch_p4_uninit();
+	csch_plug_io_uninit();
+}
+
 int csch_init_actions()
 {
 	csch_actions_init(&rnd_fgw); /* custom fgw types */
