@@ -69,6 +69,7 @@ void csch_project_free(csch_project_t *prj)
 	prj->hdr.designs.used = 0; /* rnd_project_uninit() will iterate */
 
 	csch_p4_call_project_uninit(prj);
+	vtp0_uninit(&prj->p4data);
 
 	for(n = 0; n < vtp0_len(&prj->views); n++)
 		csch_view_free(prj->views.array[n]);
@@ -255,6 +256,8 @@ csch_project_t *csch_load_project_by_sheet_name(const char *sheet_fn, int with_s
 				rnd_message(RND_MSG_WARNING, "Failed to load project file %s; creating a dummy project in memory and assuming implicit project\n", project_fn);
 		}
 		prj = csch_project_alloc();
+		if (project_fn != NULL)
+			prj->hdr.loadname = rnd_strdup(project_fn);
 	}
 	else
 		prj->dummy = 0;
