@@ -4,7 +4,8 @@
 #include "cnc_text.h"
 
 /* Place a new floater for an existing attribute, by attr key
-   If hint_xy is not NULL, it contains coords where the GUI action started */
+   If hint_xy is not NULL, it contains coords where the GUI action started, in
+   sheet coords */
 csch_text_t *csch_auto_attr_place(csch_sheet_t *sheet, csch_cgrp_t *grp, const char *key, const char *stroke, const char *atempl, csch_coord_t *hint_xy);
 
 /* Create a new attribute and place a dyntext-floater for it if stroke is
