@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  cschem - modular/flexible schematics editor - libcschem (core library)
- *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2022, 2024 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022)
  *
@@ -110,3 +110,25 @@ int csch_loclib_get_roots(csch_lib_t **root_dir_out, csch_cgrp_t **root_grp_out,
 
 	return 0;
 }
+
+long csch_loclib_reload_all(csch_sheet_t *sheet, csch_lib_root_t *libroot)
+{
+	long n, i, cnt = 0;
+
+	for(n = 0; n < libroot->roots.used; n++) {
+		csch_lib_t *root = libroot->roots.array[n];
+		if (strcmp(root->name, "<local>") == 0) {
+			for(i = root->children.used-1; i >= 0; i--) {
+				csch_lib_t *l = root->children.array[i];
+				if ((l != NULL) && (l->backend != NULL) && (l->backend->loc_refresh_from_ext != NULL)) {
+					if (l->backend->loc_refresh_from_ext(sheet, l) == 0)
+						cnt++;
+				}
+			}
+			break;
+		}
+	}
+
+	return cnt;
+}
+
