@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  cschem - modular/flexible schematics editor - sch-rnd (executable)
- *  Copyright (C) 2020,2022,2023 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2020,2022..2024 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022, Entrust in 2023)
  *
@@ -1024,7 +1024,8 @@ static void csch_wirenet_newseg2wirenets(csch_sheet_t *sheet, crmap_t *map, csch
 			wn = (csch_cgrp_t **)vtp0_get(new_wirenets, seg, 0);
 			if ((wn == NULL) || (*wn == NULL)) {
 				wn_ = csch_wirenet_new_from(sheet, line->hdr.parent);
-				wn_->hdr.selected = line->hdr.parent->hdr.selected;
+				if (line->hdr.parent != NULL)
+					wn_->hdr.selected = line->hdr.parent->hdr.selected;
 				vtp0_set(new_wirenets, seg, wn_);
 				wn = &wn_;
 			}
@@ -1067,8 +1068,14 @@ void csch_wirenet_recalc(csch_sheet_t *sheet, csch_cgrp_t *line_wn)
 
 	for(n = 0; n < new_wirenets.used; n++) {
 		if ((new_wirenets.array[n] != NULL) && (!csch_obj_is_deleted(new_wirenets.array[n]))) {
-			wirenet_remove_or_recalc(sheet, new_wirenets.array[n]);
-			csch_wirenet_recalc_wirenet_conn(sheet, new_wirenets.array[n]);
+			/* in case it now touches another wirenet, merge them */
+			csch_wirenet_recalc_merges(sheet, new_wirenets.array[n]);
+
+			/* recalc junctions and conns if not merged */
+			if (!csch_obj_is_deleted(new_wirenets.array[n])) {
+				wirenet_remove_or_recalc(sheet, new_wirenets.array[n]);
+				csch_wirenet_recalc_wirenet_conn(sheet, new_wirenets.array[n]);
+			}
 		}
 	}
 
@@ -1260,3 +1267,32 @@ int csch_is_wireline_safe(csch_sheet_t *sheet, csch_coord_t x1, csch_coord_t y1,
 	return !(res & csch_RTREE_DIR_FOUND);
 }
 
+void csch_wirenet_post_remove_checks(csch_sheet_t *sheet, csch_cgrp_t *wn, int undoable)
+{
+	htip_entry_t *e, *next;
+	int has_wires = 0;
+
+	/* check if the wirenet has anything else than floaters and connections */
+	for(e = htip_first(&wn->id2obj); e != NULL; e = htip_next(&wn->id2obj, e)) {
+		csch_chdr_t *obj = e->value;
+		if (!obj->floater && (obj->type != CSCH_CTYPE_CONN) && !csch_obj_is_junction(obj)) {
+			has_wires = 1;
+			break;
+		}
+	}
+
+	if (!has_wires) {
+		/* remove all floaters */
+		for(e = htip_first(&wn->id2obj); e != NULL; e = next) {
+			csch_chdr_t *obj = e->value;
+			next = htip_next(&wn->id2obj, e);
+			if (obj->floater) {
+				if (undoable)
+					csch_op_remove(sheet, obj);
+				else
+					csch_cnc_remove(sheet, obj);
+			}
+		}
+	}
+}
+
