@@ -9,7 +9,9 @@ typedef enum sch_rnd_ba_type_e {
 	SCH_RND_BAT_CONN_ADD,
 	SCH_RND_BAT_NET_ATTR,
 	SCH_RND_BAT_COMP_ATTR,
-	SCH_RND_BAT_TERM_ATTR
+	SCH_RND_BAT_TERM_ATTR,
+	SCH_RND_BAT_COMP_ADD,
+	SCH_RND_BAT_COMP_DEL
 } sch_rnd_ba_type_t;
 
 typedef enum sch_rnd_ba_status_e {
@@ -27,6 +29,8 @@ typedef struct sch_rnd_ba_s {
 		struct { char *comp, *term, *net; }  conn_add;
 		struct { char *net, *key, *val; }    net_attr;
 		struct { char *comp, *key, *val; }   comp_attr;
+		struct { char *comp; }   comp_add;
+		struct { char *comp; }   comp_del;
 
 		struct { char *comp, *term, *key, *val; } term_attr;
 
