@@ -44,30 +44,8 @@
 	static int tprintf(const char *fmt, ...) { return 0; }
 #endif
 
-#if 0
-int pcbdoc_ascii_test_parse(pcb_plug_io_t *ctx, pcb_plug_iot_t typ, const char *file_name, FILE *f)
-{
-	char line[256], *s;
-
-	/* first line should comply to the low level file format */
-	fgets(line, sizeof(line), f);
-
-	s = line;
-	if (*s == '|') s++;
-
-	/* every line must start with a RECORD field, the first one too */
-	if (strncmp(s,"RECORD=", 7) != 0)
-		return 0;
-
-	/* there must be a field separator */
-	if (strchr(s, '|') == NULL)
-		return 0;
-
-	return 1;
-}
-#endif
-
-static int pcbdoc_ascii_load_blocks(altium_tree_t *tree, FILE *f, long max)
+TODO("move these to pcb-rnd rnd_inclib")
+int pcbdoc_ascii_load_blocks(altium_tree_t *tree, FILE *f, long max)
 {
 	long curr = 0, next;
 
@@ -107,7 +85,7 @@ static int pcbdoc_ascii_load_blocks(altium_tree_t *tree, FILE *f, long max)
 			break;
 
 		/* by now "next" points to the first character of the next block */
-		blk = malloc(sizeof(altium_block_t) + (next-curr) + 1);
+		blk = malloc(sizeof(altium_block_t) + (next-curr) + 2);
 		if (blk == NULL) {
 			fprintf(stderr, "pcbdoc_ascii_load_blocks: failed to alloc memory\n");
 			return -1;
@@ -120,6 +98,13 @@ static int pcbdoc_ascii_load_blocks(altium_tree_t *tree, FILE *f, long max)
 			fprintf(stderr, "pcbdoc_ascii_load_blocks: can't read that many: %ld from %ld (%ld; max is %ld)\n", blk->size, curr, curr+blk->size, max);
 			return -1;
 		}
+		if (c == EOF) {
+			int last = blk->raw[blk->size-1];
+			if ((last != '\r') && (last != '\n')) {
+				blk->raw[blk->size] = '\n';
+				blk->size++;
+			}
+		}
 		blk->raw[blk->size] = '\0';
 		gdl_append(&tree->blocks, blk, link);
 /*		printf("curr=%ld next=%ld\n", curr, next);*/
@@ -212,13 +197,14 @@ int pcbdoc_ascii_parse_fields(altium_tree_t *tree, altium_record_t *rec, const c
 	return 0;
 }
 
-static int pcbdoc_ascii_parse_blocks(altium_tree_t *tree, const char *fn)
+int pcbdoc_ascii_parse_blocks(altium_tree_t *tree, const char *fn)
 {
 	altium_block_t *blk;
-	long line = 1;
+	long line = 1, idx = 0;
 
 	for(blk = gdl_first(&tree->blocks); blk != NULL; blk = gdl_next(&tree->blocks, blk)) {
 		char *s = blk->raw, *end;
+		int is_hdr;
 
 		tprintf("---blk---\n");
 
@@ -231,54 +217,45 @@ static int pcbdoc_ascii_parse_blocks(altium_tree_t *tree, const char *fn)
 				break;
 
 			/* parse the record header */
-			if (strncmp(s, "RECORD=", 7) != 0) {
-				fprintf(stderr, "First field must be record in %s:%ld\n", fn, line);
+			if ((strncmp(s, "RECORD=", 7) != 0) && (strncmp(s, "HEADER=", 7) != 0)) {
+				fprintf(stderr, "First field must be record or header in %s:%ld\n", fn, line);
 				return -1;
 			}
-			s+=7;
+			is_hdr = (s[0] == 'H');
+			if (!is_hdr)
+				s+=7;
 			end = strpbrk(s, "|\r\n");
 			if (end == NULL) {
 				fprintf(stderr, "Unterminated record in %s:%ld\n", fn, line);
 				return -1;
 			}
 			*end = '\0';
-			tprintf("rec='%s'\n", s);
-			rec = pcbdoc_ascii_new_rec(tree, s, altium_kw_AUTO);
+			tprintf("rec='%s' idx=%ld\n", s, idx);
+			if (is_hdr)
+				rec = pcbdoc_ascii_new_rec(tree, s, altium_kw_record_header);
+			else
+				rec = pcbdoc_ascii_new_rec(tree, s, altium_kw_AUTO);
 			s = end+1;
 
 			if (pcbdoc_ascii_parse_fields(tree, rec, fn, line, &s) != 0)
 				return -1;
+
+			if (is_hdr)
+				pcbdoc_ascii_new_field(tree, rec, "header", altium_kw_field_header, blk->raw+8);
+
+
+			if (!is_hdr) {
+				rec->idx = idx;
+				idx++;
+			}
+			else
+				rec->idx = -1;
 		}
 	}
 
 	return 0;
 }
 
-int pcbdoc_ascii_parse_file(rnd_design_t *hidlib, altium_tree_t *tree, const char *fn)
-{
-	FILE *f;
-	int res;
-	long filesize;
-
-	filesize = rnd_file_size(hidlib, fn);
-	if (filesize <= 0)
-		return -1;
-
-	f = rnd_fopen(hidlib, fn, "rb");
-	if (f == NULL)
-		return -1;
-
-	res = pcbdoc_ascii_load_blocks(tree, f, filesize);
-	fclose(f);
-
-	if (res != 0)
-		return -1;
-
-
-	return pcbdoc_ascii_parse_blocks(tree, fn);
-}
-
-
 void altium_tree_free(altium_tree_t *tree)
 {
 	altium_block_t *blk;
@@ -304,3 +281,5 @@ void altium_tree_free(altium_tree_t *tree)
 	}
 }
 
+
+#include "schdoc.c"
