@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - altium file format support
+ *  sch-rnd - modular/flexible schematics editor - altium file format support
  *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 Entrust in 2023)
@@ -46,6 +46,10 @@
 
 TODO("At the moment this is #included from pcbdoc_ascii.c; this would change once that code is moved to pcb-rnd's inclib")
 
+/* for records that have been executed and intentionally did not create a group;
+   for example slotted symbols for the inactive pins */
+static const csch_cgrp_t NOGRP_, *NOGRP = &NOGRP_;
+
 /* Record types:
     1: symbol placement (from lib)
     2: pin
@@ -214,7 +218,7 @@ static int altium_parse_arc(io_altium_rctx_t *rctx, altium_record_t *rec, int is
 {
 	altium_field_t *field;
 	long ri = -1, rf = 0, r2i = -1, r2f = 0;
-	double r, r2, sa = -1, ea = 0, da, cx = -1, cy = -1, cxf = 0, cyf = 0;
+	double r, r2, sa = 0, ea = 0, da, cx = -1, cy = -1, cxf = 0, cyf = 0;
 	csch_chdr_t *arc;
 	csch_cgrp_t *parent;
 	long pidx = -1;
@@ -249,6 +253,8 @@ static int altium_parse_arc(io_altium_rctx_t *rctx, altium_record_t *rec, int is
 	cy += cyf / FRAC_DIV;
 
 	parent = altium_get_parent(rctx, rec, pidx, 0, &stroke, &fill);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return -1;
 
@@ -257,8 +263,8 @@ static int altium_parse_arc(io_altium_rctx_t *rctx, altium_record_t *rec, int is
 
 	if ((ea >= 360) && (sa < 0))
 		sa = 0;
-	if ((cx < 0) || (cy < 0) || (ri < 0) || (sa < 0) || (ea < 0)) {
-		error(rec, ("altium_parse_arc(): missing coords, radius or angles\n"));
+	if ((cx < 0) || (cy < 0) || (ri < 0)) {
+		error(rec, ("altium_parse_arc(): missing coords or radius\n"));
 		return -1;
 	}
 
@@ -387,6 +393,8 @@ static int altium_parse_bezier(io_altium_rctx_t *rctx, altium_record_t *rec, vtd
 	}
 
 	parent = altium_get_parent(rctx, rec, pidx, 0, &stroke, &fill);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return -1;
 
@@ -470,6 +478,8 @@ static int altium_create_lines(io_altium_rctx_t *rctx, altium_record_t *rec, vtd
 		is_solid = alti_is_dark(aclr);
 
 	parent = altium_get_parent(rctx, rec, pidx, is_solid, &stroke, &fill);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return -1;
 
@@ -587,7 +597,7 @@ static int altium_parse_rect(io_altium_rctx_t *rctx, altium_record_t *rec, int i
 {
 	altium_field_t *field;
 	long pidx = -1, aclr = -1;
-	double x1 = -1, y1 = -1, x2 = -1, y2 = -1, x1f = 0, y1f = 0, x2f = 0, y2f = 0;
+	double x1 = 0, y1 = 0, x2 = 0, y2 = 0, x1f = 0, y1f = 0, x2f = 0, y2f = 0;
 	double xr = 0, yr = 0, xrf = 0, yrf = 0;
 	csch_chdr_t *poly;
 	csch_cgrp_t *parent;
@@ -619,15 +629,12 @@ static int altium_parse_rect(io_altium_rctx_t *rctx, altium_record_t *rec, int i
 		}
 	}
 
-	if ((x1 < 0) || (y1 < 0)) {
-		error(rec, ("altium_parse_rect(): missing location.x or location.y or corner.x or corner.y\n"));
-		return -1;
-	}
-
 	if (aclr > 0)
 		is_solid = alti_is_dark(aclr);
 
 	parent = altium_get_parent(rctx, rec, pidx, is_solid, &stroke, &fill);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return -1;
 
@@ -715,6 +722,8 @@ static int altium_parse_shref_rect(io_altium_rctx_t *rctx, altium_record_t *rec)
 	}
 
 	parent = altium_get_parent(rctx, rec, pidx, is_solid, &dummy, &dummy);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return -1;
 
@@ -810,7 +819,6 @@ static int altium_parse_attrib(io_altium_rctx_t *rctx, altium_record_t *rec, int
 	}
 
 	if (key == NULL) {
-		error(rec, ("altium_parse_attrib(): attrib has no name field\n"));
 		key = keytmp;
 		rnd_snprintf(keytmp, sizeof(keytmp), "not_named_in_record_%ld", rec->idx);
 	}
@@ -819,6 +827,8 @@ static int altium_parse_attrib(io_altium_rctx_t *rctx, altium_record_t *rec, int
 	y += yf / FRAC_DIV;
 
 	parent = altium_get_parent(rctx, rec, pidx, 0, &stroke, &fill);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return 0;
 
@@ -925,6 +935,8 @@ static int altium_parse_sign(io_altium_rctx_t *rctx, altium_record_t *rec)
 	y += yf / FRAC_DIV;
 
 	parent = altium_get_parent(rctx, rec, pidx, 0, &stroke, &fill);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return 0;
 
@@ -997,7 +1009,7 @@ static int altium_parse_net_labels(io_altium_rctx_t *rctx)
 {
 	altium_record_t *rec;
 	altium_field_t *field;
-	csch_line_t *wire;
+	csch_line_t *wire, *term = NULL;
 	csch_text_t *text;
 	csch_source_arg_t *src;
 	csch_sheet_t *sheet = rctx->alien.sheet;
@@ -1008,6 +1020,7 @@ static int altium_parse_net_labels(io_altium_rctx_t *rctx)
 		double x = -1, y = -1, xf = 0, yf = 0;
 		int ori = 0;
 		const char *name = NULL;
+		csch_coord_t x0, y0;
 
 		for(field = gdl_first(&rec->fields); field != NULL; field = gdl_next(&rec->fields, field)) {
 			switch(field->type) {
@@ -1035,8 +1048,10 @@ static int altium_parse_net_labels(io_altium_rctx_t *rctx)
 
 		/* find a wire the label is put on */
 		wire = NULL;
-		bbox.x1 = csch_alien_coord_x(&rctx->alien, x) - 1;
-		bbox.y1 = csch_alien_coord_y(&rctx->alien, y) - 1;
+		x0 = csch_alien_coord_x(&rctx->alien, x);
+		y0 = csch_alien_coord_y(&rctx->alien, y);
+		bbox.x1 = x0 - 1;
+		bbox.y1 = y0 - 1;
 		bbox.x2 = bbox.x1 + 2;
 		bbox.y2 = bbox.y1 + 2;
 		for(wire = csch_rtree_first(&it, &sheet->dsply[CSCH_DSPLY_WIRE], &bbox); wire != NULL; wire = csch_rtree_next(&it))
@@ -1044,17 +1059,34 @@ static int altium_parse_net_labels(io_altium_rctx_t *rctx)
 				break;
 
 		if (wire == NULL) {
-			error(rec, ("altium_parse_net_labels(): label not on line\n"));
-			return -1;
+			csch_comm_str_t pen;
+
+			/* net labels may be placed on terminals... */
+			for(term = csch_rtree_first(&it, &sheet->dsply[CSCH_DSPLY_HUBTERM], &bbox); wire != NULL; wire = csch_rtree_next(&it))
+				if ((term->hdr.type == CSCH_CTYPE_LINE) && (term->hdr.parent->role == CSCH_ROLE_TERMINAL))
+					break;
+
+			if (term == NULL) {
+				error(rec, ("altium_parse_net_labels(): label '%s' not on wire line or terminal line - ignoring this label\n", name));
+				return 0;
+			}
+
+
+			/* when placed on terminal, the closest thing we can do is a short stub
+			   wirenet that should not intersect with anything else */
+			pen = csch_comm_str(rctx->alien.sheet, "wire", 1);
+			wire = (csch_line_t *)csch_wirenet_draw(rctx->alien.sheet, pen, x0, y0, x0+100, y0+100);
 		}
 
-		text = (csch_text_t *)csch_alien_mktext(&rctx->alien, wire->hdr.parent, x, y, "wire");
-		text->text = rnd_strdup("%../A.name%");
-		text->dyntext = 1;
-		text->hdr.floater = 1;
-		text->spec_rot = ori * 90;
-		src = csch_attrib_src_c(rctx->fn, rec->idx+1, 0, NULL);
-		csch_attrib_set(&wire->hdr.parent->attr, CSCH_ATP_USER_DEFAULT, "name", name, src, NULL);
+		if (wire != NULL) {
+			text = (csch_text_t *)csch_alien_mktext(&rctx->alien, wire->hdr.parent, x, y, "wire");
+			text->text = rnd_strdup("%../A.name%");
+			text->dyntext = 1;
+			text->hdr.floater = 1;
+			text->spec_rot = ori * 90;
+			src = csch_attrib_src_c(rctx->fn, rec->idx+1, 0, NULL);
+			csch_attrib_set(&wire->hdr.parent->attr, CSCH_ATP_USER_DEFAULT, "name", name, src, NULL);
+		}
 	}
 
 	return 0;
@@ -1531,6 +1563,10 @@ TODO("figure the other directions");
 	src = csch_attrib_src_c(rctx->fn, line, 0, NULL);
 	csch_alien_mkpin_line(&rctx->alien, src, sym, x1, 0, x1+1, 0);
 
+	if (style == 3) {
+		src = csch_attrib_src_c(rctx->fn, line, 0, NULL);
+		csch_alien_mkpin_line(&rctx->alien, src, sym, x2, 0, x2-1, 0);
+	}
 
 	left = right = 0;
 	switch(iotype) {
@@ -1600,6 +1636,8 @@ static int altium_parse_shref_port(io_altium_rctx_t *rctx, altium_record_t *rec)
 	}
 
 	parent = altium_get_parent(rctx, rec, pidx, 0, &dummy, &dummy);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return -1;
 
@@ -1729,7 +1767,7 @@ static int altium_parse_note(io_altium_rctx_t *rctx, altium_record_t *rec, int h
 {
 	altium_field_t *field;
 	csch_cgrp_t *parent;
-	long x1 = -1, y1 = -1, x2 = -1, y2 = -1, aclr = -1, rad = 5, fonth = 3000, pidx = -1;
+	long x1 = 0, y1 = 0, x2 = -1, y2 = -1, aclr = -1, rad = 5, fonth = 3000, pidx = -1;
 	int is_solid = 0, just = 0, ori = 0;
 	char *stroke, *fill, *textstr = NULL;
 	const char *opartid = NULL, *opartdsp = NULL;
@@ -1754,10 +1792,6 @@ static int altium_parse_note(io_altium_rctx_t *rctx, altium_record_t *rec, int h
 		}
 	}
 
-	if ((x1 < 0) || (y1 < 0) ) {
-		error(rec, ("altium_parse_note(): note rectangle without coords (start)\n"));
-		return -1;
-	}
 	if (allow_multiline && ((x2 < 0) || (y2 < 0))) {
 		error(rec, ("altium_parse_note(): note rectangle without coords (end)\n"));
 		return -1;
@@ -1767,6 +1801,8 @@ static int altium_parse_note(io_altium_rctx_t *rctx, altium_record_t *rec, int h
 		is_solid = alti_is_dark(aclr);
 
 	parent = altium_get_parent(rctx, rec, pidx, is_solid, &stroke, &fill);
+	if (parent == NOGRP)
+		return 0;
 	if (parent == NULL)
 		return -1;
 
@@ -2119,7 +2155,7 @@ static void *altium_prealloc_pin(io_altium_rctx_t *rctx, altium_record_t *rec)
 	altium_field_t *field;
 	altium_record_t *prec;
 	const char *designator = NULL, *name = NULL, *opartid = NULL, *opartdsp = NULL;
-	double x = 1, y = -1, xf = 0, yf = 0, len = -1, lenf = 0, startx, starty, endx, endy, nameshx;
+	double x = 0, y = 0, xf = 0, yf = 0, len = -1, lenf = 0, startx, starty, endx, endy, nameshx;
 	long pidx = -1, cong = -1, oute = 0, outr = 0, ine = 0, inr = 0, ele = 0;
 	long r = 3, fonth = 3000;
 	int textrot, textydir;
@@ -2147,8 +2183,8 @@ static void *altium_prealloc_pin(io_altium_rctx_t *rctx, altium_record_t *rec)
 		}
 	}
 
-	if ((x < 0) || (y < 0) || (len < 0)) {
-		error(rec, ("altium_prealloc_pin(): missing coordinate or length\n"));
+	if (len < 0) {
+		error(rec, ("altium_prealloc_pin(): length\n"));
 		return NULL;
 	}
 
@@ -2173,7 +2209,7 @@ static void *altium_prealloc_pin(io_altium_rctx_t *rctx, altium_record_t *rec)
 	}
 
 	if (alti_slot_skip(parent, opartid, opartdsp))
-		return NULL;
+		return NOGRP;
 
 	/* orientation */
 	switch(cong & 0x03) {
