@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - gEDA file format support
+ *  sch-rnd - modular/flexible schematics editor - gEDA file format support
  *  Copyright (C) 2022, 2023 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022, Entrust in 2023)
@@ -51,6 +51,7 @@
 #include <librnd/core/math_helper.h>
 #include <librnd/core/rotate.h>
 #include <librnd/core/safe_fs.h>
+#include <librnd/core/paths.h>
 #include <sch-rnd/buffer.h>
 
 #include <plugins/lib_alien/read_helper.h>
@@ -810,8 +811,12 @@ static void init_sym_hash(read_ctx_t *ctx)
 
 	htsp_init(&ctx->sym_hash, strhash, strkeyeq);
 
-	for(ci = rnd_conflist_first((rnd_conflist_t *)&io_geda_conf.plugins.io_geda.library_search_paths); ci != NULL; ci = rnd_conflist_next(ci))
-		map_dir(ctx, ci->val.string[0], cwd, &tmp);
+	for(ci = rnd_conflist_first((rnd_conflist_t *)&io_geda_conf.plugins.io_geda.library_search_paths); ci != NULL; ci = rnd_conflist_next(ci)) {
+		char *path = rnd_build_fn(&ctx->sheet->hidlib, ci->val.string[0]);
+		/*rnd_trace("io_geda: path resovle '%s' -> '%s'\n", ci->val.string[0], path);*/
+		map_dir(ctx, path, cwd, &tmp);
+		free(path);
+	}
 
 	free(cwd);
 	gds_uninit(&tmp);
@@ -932,7 +937,8 @@ static void flush_last_comp(read_ctx_t *ctx)
 
 				last_comp_move_fields(ctx, newo, ctx->last_comp);
 
-				ox = ds->grp->hdr.bbox.x1;
+/* This breaks. We shouldn't ever use the bbox, but symbol's 0;0 */
+				ox = ds->grp->hdr.bbox.x1 * 0;
 				oy = ds->grp->hdr.bbox.y1;
 				if ((ox != 0) || (oy != 0)) {
 					if (newo->spec_rot != 0) {
@@ -944,6 +950,7 @@ static void flush_last_comp(read_ctx_t *ctx)
 					newo->x -= ox;
 					newo->y -= oy;
 				}
+
 			}
 			else
 				error(ctx, ("Failed to load library symbol called '%s'\n", ctx->last_comp_name));
@@ -985,7 +992,7 @@ static int read_comp(read_ctx_t *ctx, csch_cgrp_t *parent)
 	ctx->last_comp->y = y;
 	ctx->last_comp->hdr.lock = !selectable;
 	ctx->last_comp_name = rnd_strdup(name);
-	ctx->last_comp_rot = -angle;
+	ctx->last_comp_rot = angle;
 	ctx->last_comp_mir = mirror;
 	ctx->last = &ctx->last_comp->hdr;
 
@@ -1002,7 +1009,7 @@ static int read_attr(read_ctx_t *ctx)
 	if (ctx->last != NULL)
 		parent = csch_alien_attr_grp(&ctx->alien, &ctx->last);
 	else
-	parent = &ctx->sheet->direct;
+		parent = &ctx->sheet->direct;
 
 	for(;;) {
 		int ws, cmd;
@@ -1012,7 +1019,8 @@ static int read_attr(read_ctx_t *ctx)
 			/* if last object already had attributes in a {} block, any new
 			   attribute defined on the same level is really attribute of the parent
 			   group of the last object; until a new object is created */
-			ctx->last = &ctx->last->parent->hdr;
+			if (ctx->last != NULL)
+				ctx->last = &ctx->last->parent->hdr;
 			break;
 		}
 		if (cmd != 'T') {
