@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - alien file format helpers
+ *  sch-rnd - modular/flexible schematics editor - alien file format helpers
  *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022)
@@ -256,7 +256,7 @@ csch_chdr_t *csch_alien_mkarc(csch_alien_read_ctx_t *ctx, csch_cgrp_t *parent, d
 {
 	csch_arc_t *arc;
 
-	cx = CRDX(cx); cy = CRDX(cy);
+	cx = CRDX(cx); cy = CRDY(cy);
 	r = CRD(r);
 
 	arc = csch_arc_alloc(ctx->sheet, parent, csch_oid_new(ctx->sheet, parent));
