@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - non-graphical sheet helpers
+ *  sch-rnd - modular/flexible schematics editor - non-graphical sheet helpers
  *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 Entrust in 2023)
@@ -174,42 +174,107 @@ static csch_cgrp_t *get_src_oid(const csch_sheet_t *sheet_, long *src_oid)
 	return cgrp;
 }
 
-int sch_ngrp_compile_sheet(csch_abstract_t *dst, int viewid, const csch_sheet_t *src, sch_ngrp_t *ctx)
+int sch_ngrp_compile_sheet(csch_abstract_t *dst, int viewid, csch_hier_path_t *hpath, const csch_sheet_t *src, sch_ngrp_t *ctx)
 {
 	sch_ngrp_conn_t *cn;
 	sch_ngrp_comp_t *c;
 	sch_ngrp_net_t *n;
-	long src_oid = 0;
+	long src_oid = 0, tmp_len;
+	csch_ascope_t scope;
+	gds_t tmp = {0};
 
+	gds_append_len(&tmp, hpath->prefix.array, hpath->prefix.used);
+	tmp_len = hpath->prefix.used;
 
 	/* create components */
 	for(c = ctx->comps; c != NULL; c = c->next) {
-		csch_acomp_t *acomp = csch_acomp_get(dst, c->name, 1);
+		csch_acomp_t *acomp = csch_acomp_new(dst, dst->hroot, CSCH_ASCOPE_GLOBAL, c->name, c->name);
 		csch_cgrp_t *cgrp = get_src_oid(src, &src_oid);
 
 		csch_compile_add_source(cgrp, &acomp->hdr);
 		comp_add_attrs(src, &acomp->hdr, c->attr_head);
+		c->ahdr = &acomp->hdr;
 	}
 
 	/* create nets */
 	for(n = ctx->nets; n != NULL; n = n->next) {
-		csch_anet_t *anet = csch_anet_get(dst, n->name, 1, 1);
-		csch_cgrp_t *cgrp = get_src_oid(src, &src_oid);
+		csch_anet_t *anet;
+		csch_cgrp_t *cgrp;
+		const char *orig_name = n->name;
 
+		scope = csch_split_name_scope(&orig_name);
+		tmp.used = tmp_len;
+		gds_append_str(&tmp, orig_name);
+
+		anet = csch_anet_get_at(dst, hpath->hlev, scope, orig_name);
+		if (anet == NULL)
+			anet = csch_anet_new(dst, hpath->hlev, scope, tmp.array, orig_name, 1);
+
+		cgrp = get_src_oid(src, &src_oid);;
 		csch_compile_add_source(cgrp, &anet->hdr);
 		comp_add_attrs(src, &anet->hdr, n->attr_head);
 	}
 
 	/* create connections */
 	for(cn = ctx->conns; cn != NULL; cn = cn->next) {
-		csch_anet_t *anet = csch_anet_get(dst, cn->netname, 1, 1);
-		csch_acomp_t *acomp = csch_acomp_get(dst, cn->comp, 1);
-		csch_aport_t *aport = csch_aport_get(dst, acomp, cn->port, 1);
+		csch_acomp_t *acomp;
+		csch_aport_t *aport;
 		csch_cgrp_t *cgrp = get_src_oid(src, &src_oid);
+		csch_anet_t *anet;
+		const char *orig_name;
+
+
+		acomp = csch_acomp_get(dst, cn->comp);
+		if (acomp == NULL)
+			acomp = csch_acomp_new(dst, dst->hroot, CSCH_ASCOPE_GLOBAL, cn->comp, cn->comp);
+		aport = csch_aport_get(dst, acomp, cn->port, 1);
+
+		orig_name = cn->netname;
+		scope = csch_split_name_scope(&orig_name);
+		tmp.used = tmp_len;
+		gds_append_str(&tmp, orig_name);
+
+		anet = csch_anet_get_at(dst, hpath->hlev, scope, orig_name);
+		if (anet == NULL)
+			anet = csch_anet_new(dst, hpath->hlev, scope, tmp.array, orig_name, 1);
+
 		csch_compile_add_source(cgrp, &aport->hdr);
 		csch_compile_connect_net_to(&anet, &aport->hdr, 1);
 	}
 
+#if 0
+	/* descend into hierarchic subsheets */
+	for(c = ctx->comps; c != NULL; c = c->next) {
+		csch_acomp_t *acomp = (csch_acomp_t *)c->ahdr;
+		const sch_ngrp_attrib_t *s;
+		const char *chuuid = NULL, *chname = NULL, *chpath = NULL;
+		csch_sheet_t *child_out = NULL;
+		int hres;
+
+		c->ahdr = NULL;
+
+		for(s = c->attr_head; s != NULL; s = s->next) {
+			if ((s->key[0] == 'c') && (strncmp(s->key, "cschem/child/", 13) == 0)) {
+				const char *key = s->key + 13;
+				if (strcmp(key, "uuid") == 0) chuuid = s->val;
+				else if (strcmp(key, "name") == 0) chname = s->val;
+				else if (strcmp(key, "path") == 0) chpath = s->val;
+			}
+		}
+		
+		hres = csch_hier_find_child(src, chuuid, chname, chpath, &child_out, 0);
+		if (hres < 0)
+			return -1;
+
+		if (hres == 1) {
+			TODO("DESCEND: call csch_compile_descend()");
+			TODO("then bind ports like in compile.c compile_symbol_child_sheet() at ''Bind sheet ref sym terminals to child sheet terminals''");
+		}
+	}
+#endif
+
+
+	gds_uninit(&tmp);
 	return 0;
 }
 
