@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - ngspice to high level sim glue
+ *  sch-rnd - modular/flexible schematics editor - ngspice to high level sim glue
  *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 Entrust in 2023)
@@ -47,6 +47,7 @@
 #include <librnd/core/plugins.h>
 #include <librnd/core/misc_util.h>
 #include <librnd/core/safe_fs.h>
+#include <librnd/core/compat_fs_dir.h>
 
 #include <plugins/sim/sim.h>
 #include <plugins/sim/sim_conf.h>
@@ -80,7 +81,7 @@ static sch_sim_setup_t *se_ngspice_alloc(void)
 		return NULL;
 
 
-	if (sch_sim_mktempdir(dsg, &setup->tmpdir, "sch-rnd-sim") != 0) {
+	if (rnd_mktempdir(dsg, &setup->tmpdir, "sch-rnd-sim") != 0) {
 		free(setup);
 		return NULL;
 	}
@@ -106,7 +107,7 @@ static void se_ngspice_free(sch_sim_setup_t *setup)
 	if (sch_sim_conf.plugins.sim.preserve_tmp)
 		rnd_message(RND_MSG_INFO, "Not removing sim tmp dir %s\n(as requested by the config node plugins/sim/preserve_tmp)\n", setup->tmpdir);
 	else
-		sch_sim_rmtempdir(dsg, &setup->tmpdir);
+		rnd_rmtempdir(dsg, &setup->tmpdir);
 
 	gds_uninit(&setup->read_buff);
 
@@ -115,26 +116,16 @@ static void se_ngspice_free(sch_sim_setup_t *setup)
 
 #include <librnd/hid/hid.h>
 #include <librnd/hid/hid_init.h>
+#include <librnd/hid/hid_export.h>
 #include <librnd/core/event.h>
+
 static int sim_export(rnd_design_t *dsg, char *exporter_name, int argc, char *args[])
 {
-	char **a;
-
-	rnd_exporter = rnd_hid_find_exporter(exporter_name);
-	if (rnd_exporter == NULL) {
-		rnd_message(RND_MSG_ERROR, "sim_ngspice circuit export failed: spice exporter not found.\n");
+	if (rnd_hid_export_using(dsg, exporter_name, argc, args) < 0) {
+		rnd_message(RND_MSG_ERROR, "sim_ngspice circuit export failed: spice exporter not found (see above).\n");
 		return -1;
 	}
 
-	/* call the exporter */
-	a = args;
-	rnd_event(dsg, RND_EVENT_EXPORT_SESSION_BEGIN, NULL);
-	rnd_exporter->parse_arguments(rnd_exporter, &argc, &a);
-	rnd_exporter->do_export(rnd_exporter, dsg, NULL, NULL);
-	rnd_event(dsg, RND_EVENT_EXPORT_SESSION_END, NULL);
-
-	rnd_exporter = NULL;
-
 	return 0;
 }
 
@@ -157,7 +148,10 @@ static int se_ngspice_set_global(csch_abstract_t *abst, int eng_prio, const char
 		gds_append_str(&tmp, ".temp ");
 		gds_append_str(&tmp, val->val.str);
 
-		comp = csch_acomp_get(abst, "sim_ngspice_env_temp", 1);
+		comp = csch_acomp_get(abst, "sim_ngspice_env_temp");
+		if (comp == NULL)
+			csch_acomp_new(abst, abst->hroot, CSCH_ASCOPE_GLOBAL, "sim_ngspice_env_temp", "sim_ngspice_env_temp");
+		
 
 		src = csch_attrib_src_p("sim_ngspice", "'temp' modifier");
 		csch_attrib_set(&comp->hdr.attr, eng_prio + CSCH_PRI_PLUGIN_NORMAL, "spice/command", tmp.array, src, NULL);
@@ -601,12 +595,19 @@ static int on_load(fgw_obj_t *obj, const char *filename, const char *opts)
 	return 0;
 }
 
+static int on_unload(fgw_obj_t *obj)
+{
+	target_spice->unload(obj);
+	return 0;
+}
+
+
 static const fgw_eng_t fgw_sim_ngspice_eng = {
 	"target_sim_ngspice",
 	csch_c_call_script,
 	NULL,
 	on_load,
-	NULL /* unload */
+	on_unload
 };
 
 int pplg_check_ver_sim_ngspice(int ver_needed) { return 0; }
