@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - standard device mapper
+ *  sch-rnd - modular/flexible schematics editor - standard device mapper
  *  Copyright (C) 2019,2020,2022,2023 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,10 +33,12 @@ fgw_error_t devmap_term2port(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	const char *pname;
 	const char *slot;
 	gds_t tmp;
+	csch_hier_path_t *hpath;
 
 	CSCH_HOOK_CONVARG(1, FGW_STR,  devmap_term2port, pname = argv[1].val.cstr);
 /*	CSCH_HOOK_CONVARG(2, FGW_AOBJ, devmap_term2port, comp = fgw_aobj(&argv[2]));*/
 	CSCH_HOOK_CONVARG(3, FGW_COBJ, devmap_term2port, term = fgw_cobj(&argv[3]));
+	CSCH_HOOK_CONVARG(4, FGW_HPATH, devmap_term2port, hpath = fgw_hpath(&argv[4]));
 
 	sym = term->hdr.parent;
 	slot = csch_attrib_get_str(&sym->attr, "-slot");
