@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - standard cschem attributes
+ *  sch-rnd - modular/flexible schematics editor - standard cschem forge
  *  Copyright (C) 2022, 2023 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022)
@@ -145,6 +145,7 @@ static int forge_sub(csch_ahdr_t *aobj, const char *args, int prio, int global,
 	if (prio > a->prio) {
 		csch_source_arg_t *src = csch_attrib_src_pa(aobj, srctxt, "forge", "inst: *sub*");
 		csch_attrib_append_src(a, prio, src, 1);
+		csch_attr_src_free(src);
 		return 0;
 	}
 
@@ -167,6 +168,7 @@ static int forge_sub(csch_ahdr_t *aobj, const char *args, int prio, int global,
 	{
 		csch_source_arg_t *src = csch_attrib_src_pa(aobj, srctxt, "forge", "inst: *sub*");
 		csch_attrib_append_src(a, prio, src, 0);
+		csch_attr_src_free(src);
 	}
 	return 0;
 }
@@ -240,6 +242,7 @@ static int forge_copy(csch_ahdr_t *aobj, const char *args, int prio, int where,
 	if (prio > dst->prio) {
 		csch_source_arg_t *src = csch_attrib_src_pa(aobj, srctxt, "forge", "inst: prepend/append/copy");
 		csch_attrib_append_src(dst, prio, src, 1);
+		csch_attr_src_free(src);
 		return 0;
 	}
 
@@ -298,6 +301,7 @@ static int forge_copy(csch_ahdr_t *aobj, const char *args, int prio, int where,
 	{
 		csch_source_arg_t *src = csch_attrib_src_pa(aobj, srctxt, "forge", "inst: prepend/append/copy");
 		csch_attrib_append_src(dst, prio, src, 0);
+		csch_attr_src_free(src);
 	}
 
 	return 0;
