@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - attr. transf. for PCB workflow
+ *  sch-rnd - modular/flexible schematics editor - spice target
  *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 Entrust in 2022)
@@ -52,6 +52,7 @@
 #include <libcschem/util_compile.h>
 #include <libcschem/util_loclib.h>
 #include <libcschem/util_lib_fs.h>
+#include <libcschem/hierarchy.h>
 
 #include <plugins/sch_dialogs/quick_attr_util.h>
 #include <plugins/lib_target/lib_target.h>
@@ -176,10 +177,12 @@ fgw_error_t target_spice_sym2comp(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
 	csch_cgrp_t *sym;
 	const char *sname, *slot;
+	csch_hier_path_t *hpath;
 	gds_t tmp;
 
 	CSCH_HOOK_CONVARG(1, FGW_STR,  target_spice_sym2comp, sname = argv[1].val.cstr);
 	CSCH_HOOK_CONVARG(2, FGW_COBJ, target_spice_sym2comp, sym = fgw_cobj(&argv[2]));
+	CSCH_HOOK_CONVARG(3, FGW_HPATH, target_spice_sym2comp, hpath = fgw_hpath(&argv[3]));
 
 	slot = get_slot(sym);
 	if (slot == NULL)
@@ -187,6 +190,14 @@ fgw_error_t target_spice_sym2comp(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 
 	/* output port name is slot/input port name */
 	gds_init(&tmp);
+	if (hpath->prefix.used > 0) {
+		gds_append(&tmp, sname[0]);
+		gds_append_len(&tmp, hpath->prefix.array, hpath->prefix.used);
+	}
+	gds_append_str(&tmp, sname);
+	gds_append_str(&tmp, SLOTSEP);
+	gds_append_str(&tmp, slot);
+	gds_append(&tmp, '\n');
 	gds_append_str(&tmp, sname);
 	gds_append_str(&tmp, SLOTSEP);
 	gds_append_str(&tmp, slot);
@@ -299,7 +310,9 @@ static csch_acomp_t *make_acomp_for_model_card(spicelib_ctx_t *ctx, csch_acomp_t
 	if (modcompname == NULL)
 		modcompname = freeme = rnd_concat("_spice_model_card_", model_name, NULL);
 
-	modcomp = csch_acomp_get(comp->hdr.abst, modcompname, 1);
+	modcomp = csch_acomp_get(comp->hdr.abst, modcompname);
+	if (modcomp == NULL)
+		modcomp = csch_acomp_new(comp->hdr.abst, comp->hdr.abst->hroot, CSCH_ASCOPE_GLOBAL, modcompname, modcompname);
 	if (modcomp == NULL)
 		return NULL;
 
@@ -354,8 +367,9 @@ fgw_error_t target_spice_compile_component1(fgw_arg_t *res, int argc, fgw_arg_t
 	csch_attrib_t *portmap, *amodel;
 	char *tmp;
 	csch_source_arg_t *src;
-	const char *model_card;
+	const char *model_card, *shprefix;
 	int is_subckt = 0;
+	gds_t hprefix = {0};
 
 
 	CSCH_HOOK_CONVARG(1, FGW_AOBJ, std_cschem_comp_update, comp = fgw_aobj(&argv[1]));
@@ -418,14 +432,23 @@ fgw_error_t target_spice_compile_component1(fgw_arg_t *res, int argc, fgw_arg_t
 
 /*	rnd_trace("spice rename: name='%s' prefix='%s'\n", name, prefix);*/
 
+	/* compute hierarchic prefix */
+	if (comp->hlev->parent != NULL) {
+		csch_hier_build_path(&hprefix, comp->hlev);
+		shprefix = hprefix.array;
+	}
+	else
+		shprefix ="";
+
 	src = csch_attrib_src_p("target_spice", "instance prefix from spice/prefix");
 	slot = get_aslot(comp);
 	if (slot != NULL)
-		tmp = rnd_concat(prefix, "_", name, SLOTSEP, slot, NULL);
+		tmp = rnd_concat(prefix, "_", shprefix, name, SLOTSEP, slot, NULL);
 	else
-		tmp = rnd_concat(prefix, "_", name, NULL);
+		tmp = rnd_concat(prefix, "_", shprefix, name, NULL);
 
 	csch_attrib_set(&comp->hdr.attr, eng->eprio + CSCH_PRI_PLUGIN_NORMAL, "display/name", tmp, src, NULL);
+	gds_uninit(&hprefix);
 	free(tmp);
 
 	return 0;
@@ -606,7 +629,7 @@ static void bridge_insert(spicelib_ctx_t *ctx, csch_abstract_t *abst, csch_aport
 	tmp->used = 0;
 	rnd_append_printf(tmp, "br_%s_%ld", modelsuff, puid);
 	netname = tmp->array;
-	new_net = csch_anet_get(abst, netname, 1, 1);
+	new_net = csch_anet_new(abst, NULL, CSCH_ASCOPE_GLOBAL, netname, netname, 1);
 
 /*rnd_trace("  net: '%s' %p\n", netname, new_net);*/
 	old_net = port->conn.net;
@@ -813,12 +836,20 @@ static int on_load(fgw_obj_t *obj, const char *filename, const char *opts)
 	return 0;
 }
 
+static int on_unload(fgw_obj_t *obj)
+{
+	spicelib_ctx_t *ctx = obj->script_data;
+	ldch_uninit(&ctx->spicelibs);
+	free(ctx);
+	return 0;
+}
+
 static const fgw_eng_t fgw_target_spice_eng = {
 	"target_spice",
 	csch_c_call_script,
 	NULL,
 	on_load,
-	NULL, /* on_unload */
+	on_unload
 };
 
 static rnd_action_t spicelib_action_list[] = {
