@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - sch-rnd (executable)
+ *  sch-rnd - modular/flexible schematics editor - sch-rnd (executable)
  *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022)
@@ -443,8 +443,8 @@ fgw_error_t csch_act_BufferLoad(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	return 0;
 }
 
-static const char csch_acts_BufferChk[] = "BufferChk([group|symbol|sheet, [filename], [fmt])";
-static const char csch_acth_BufferChk[] = "Chk content of buffer; if first arg is symbol, the first symbol group is saved, if sheet the whole buffer is saved as sheet\n";
+static const char csch_acts_BufferChk[] = "BufferChk(num)";
+static const char csch_acth_BufferChk[] = "Returns whether buffer num is active or not\n";
 fgw_error_t csch_act_BufferChk(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
 	int expected;
@@ -453,8 +453,8 @@ fgw_error_t csch_act_BufferChk(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	return 0;
 }
 
-static const char csch_acts_BufferSwitch[] = "BufferSwitch([group|symbol|sheet, [filename], [fmt])";
-static const char csch_acth_BufferSwitch[] = "Switch content of buffer; if first arg is symbol, the first symbol group is saved, if sheet the whole buffer is saved as sheet\n";
+static const char csch_acts_BufferSwitch[] = "BufferSwitch(num)";
+static const char csch_acth_BufferSwitch[] = "Activates buffer num\n";
 fgw_error_t csch_act_BufferSwitch(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
 	int bn;
