@@ -1,8 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - sch-rnd (executable)
- *  Copyright (C) 2022, 2023 Tibor 'Igor2' Palinkas
+ *  sch-rnd - modular/flexible schematics editor - sch-rnd (executable)
+ *  Copyright (C) 2022, 2023, 2024 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022, Entrust in 2023)
  *
@@ -39,9 +39,10 @@
 #include <libcschem/operation.h>
 #include <libcschem/project.h>
 #include <libcschem/plug_io.h>
-#include <libcschem/util_path.h>
+#include <libcschem/util_loclib.h>
 #include <librnd/core/actions.h>
 #include <librnd/hid/hid.h>
+#include <librnd/hid/hid_export.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/compat_fs.h>
 #include <librnd/core/compat_lrealpath.h>
@@ -53,6 +54,16 @@
 #include "multi.h"
 #include "project.h"
 
+/* Return 1 if any of the currently loaded sheets have changes on them */
+static int any_sheet_changed(void)
+{
+	csch_sheet_t *sheet;
+	for(sheet = gdl_first(&rnd_designs); sheet != NULL; sheet = sheet->hidlib.link.next)
+		if (sheet->changed)
+			return 1;
+
+	return 0;
+}
 
 static const char csch_acts_Quit[] = "Quit()";
 static const char csch_acth_Quit[] = "Quits sch-rnd after confirming.";
@@ -65,8 +76,7 @@ static fgw_error_t csch_act_Quit(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	if ((force != NULL) && (rnd_strcasecmp(force, "force") == 0))
 		exit(0);
 
-TODO("need to check all sheets, not just the current one")
-	if (!sheet->changed || (rnd_hid_message_box(RND_ACT_DESIGN, "warning", "Close: lose data", "Exiting sch-rnd will lose unsaved sheet modifications.", "cancel", 0, "ok", 1, NULL) == 1))
+	if (!any_sheet_changed() || (rnd_hid_message_box(RND_ACT_DESIGN, "warning", "Close: lose data", "Exiting sch-rnd will lose unsaved sheet modifications.", "cancel", 0, "ok", 1, NULL) == 1))
 		sch_rnd_quit_app();
 
 	RND_ACT_IRES(-1);
@@ -92,25 +102,10 @@ static const char csch_acth_SymlibReloadAllLocal[] = "Replace all local symbols
 static fgw_error_t csch_act_SymlibReloadAllLocal(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
 	csch_sheet_t *sheet = CSCH_ACT_SHEET;
-	long cnt = 0, n, i;
-	csch_lib_root_t *libroot;
 	csch_lib_master_t *master = csch_lib_get_master("symbol", 0);
+	long cnt;
 
-	libroot = sheet->local_libs.array[master->uid];
-	for(n = 0; n < libroot->roots.used; n++) {
-		csch_lib_t *root = libroot->roots.array[n];
-		if (strcmp(root->name, "<local>") == 0) {
-			for(i = root->children.used-1; i >= 0; i--) {
-				csch_lib_t *l = root->children.array[i];
-				if ((l != NULL) && (l->backend != NULL) && (l->backend->loc_refresh_from_ext != NULL)) {
-					if (l->backend->loc_refresh_from_ext(sheet, l) == 0)
-						cnt++;
-				}
-			}
-			break;
-		}
-	}
-
+	cnt = csch_loclib_reload_all(sheet, sheet->local_libs.array[master->uid]);
 	rnd_message(RND_MSG_INFO, "Reloaded %ld local lib symbol(s) from external libs\n", cnt);
 
 	rnd_event(&sheet->hidlib, CSCH_EVENT_LIBRARY_CHANGED, NULL);
@@ -118,6 +113,19 @@ static fgw_error_t csch_act_SymlibReloadAllLocal(fgw_arg_t *res, int argc, fgw_a
 	return 0;
 }
 
+static const char csch_acts_HlibraryRehash[] = "HlibraryRehash()";
+static const char csch_acth_HlibraryRehash[] = "Rebuild the in-memory tree of hierarchic sheet libraries";
+static fgw_error_t csch_act_HlibraryRehash(fgw_arg_t *res, int argc, fgw_arg_t *argv)
+{
+	csch_sheet_t *sheet = CSCH_ACT_SHEET;
+	csch_lib_master_t *master = csch_lib_get_master("hlibrary", 1);
+
+	csch_lib_clear_sheet_lib(sheet, master->uid);
+	csch_lib_add_all(sheet, master, &conf_core.rc.library_search_paths, 1);
+	rnd_event(&sheet->hidlib, CSCH_EVENT_LIBRARY_CHANGED, NULL);
+	RND_ACT_IRES(0);
+	return 0;
+}
 
 static const char csch_acts_Revert[] = "Revert([sheet|project])";
 static const char csch_acth_Revert[] = "Revert to on-disk version of file(s). Default target is sheet.";
@@ -171,9 +179,9 @@ static void prj_append_sheet(csch_project_t *prj, csch_sheet_t *sheet, const cha
 		char *val = NULL;
 
 		if (prj->hdr.fullpath == NULL)
-			val = rnd_strdup(csch_basename(sheet->hidlib.fullpath));
+			val = rnd_strdup(rnd_basename(sheet->hidlib.fullpath));
 		else if ((sheet->hidlib.fullpath != NULL) && (*sheet->hidlib.fullpath != '<'))
-			val = csch_relative_path_files(sheet->hidlib.fullpath, prj->hdr.fullpath);
+			val = rnd_relative_path_files(sheet->hidlib.fullpath, prj->hdr.fullpath);
 		else
 			rnd_message(RND_MSG_ERROR, "Failed to append sheet to project file's %s;\ntry to save the sheet first so it gets a file name!\n", listpath);
 
@@ -244,12 +252,18 @@ fgw_error_t csch_act_New(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 
 	rnd_multi_switch_to(NULL); /* empty conf so we can overwrite */
 
-	ns = sch_rnd_multi_new_empty(path);
+	/* NOTE: this may be a contradiction to use curr->hidlib.project instead of
+	         NULL: create a new unlisted sheet from menu, name it /tmp/foo.rs
+	         and it will be put in the current project in memory, but not in
+	         the project file (since it's unlisted). So this sheet is now using
+	         the wrong project.lht that it wouldn't use when loaded */
+	ns = sch_rnd_multi_new_empty_in_prj((csch_project_t *)curr->hidlib.project, path);
 	free(path);
 
 	ns->changed = 1; /* sheet is unsaved when created */
 
 	prj = (csch_project_t *)ns->hidlib.project;
+
 	switch(*type) {
 		case 'r':
 			ns->stype = CSCH_SHTY_ROOT;
@@ -312,7 +326,7 @@ fgw_error_t csch_act_ProjectNew(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		char *nam, *bn;
 
 		rnd_multi_switch_to(NULL); /* empty conf so we can overwrite */
-		ns = sch_rnd_multi_new_empty_in_prj(prj);
+		ns = sch_rnd_multi_new_empty_in_prj(prj, NULL);
 		ns->changed = 1; /* sheet is unsaved when created */
 
 		nam = rnd_strdup(prj->hdr.fullpath);
@@ -587,8 +601,10 @@ static rnd_action_t file_action_list[] = {
 	{"Quit", csch_act_Quit, csch_acth_Quit, csch_acts_Quit},
 	{"SymlibRehash", csch_act_SymlibRehash, csch_acth_SymlibRehash, csch_acts_SymlibRehash},
 	{"SymlibReloadAllLocal", csch_act_SymlibReloadAllLocal, csch_acth_SymlibReloadAllLocal, csch_acts_SymlibReloadAllLocal},
+	{"HlibraryRehash", csch_act_HlibraryRehash, csch_acth_HlibraryRehash, csch_acts_HlibraryRehash},
 	{"Revert", csch_act_Revert, csch_acth_Revert, csch_acts_Revert},
 	{"New", csch_act_New, csch_acth_New, csch_acts_New},
+	{"Export", rnd_act_Export, rnd_acth_Export, rnd_acts_Export},
 	{"ProjectNew", csch_act_ProjectNew, csch_acth_ProjectNew, csch_acts_ProjectNew},
 	{"ProjectLoadPartial", csch_act_ProjectLoadPartial, csch_acth_ProjectLoadPartial, csch_acts_ProjectLoadPartial},
 	{"ProjectSheetType", csch_act_ProjectSheetType, csch_acth_ProjectSheetType, csch_acts_ProjectSheetType},
