@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - sch-rnd (executable)
+ *  sch-rnd - modular/flexible schematics editor - sch-rnd (executable)
  *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022)
@@ -54,6 +54,7 @@
 long sch_rnd_font_score_debug;
 
 #include "font_rnd.c"
+#include "font_act.c"
 
 void sch_rnd_font_text_update(csch_sheet_t *sheet, csch_text_t *t, csch_cpen_t *pen)
 {
