@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - sch-rnd (executable)
+ *  sch-rnd - modular/flexible schematics editor - sch-rnd (executable)
  *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022)
@@ -31,7 +31,7 @@
 #include <ctype.h>
 #include <genht/htsp.h>
 #include <genht/hash.h>
-#include <librnd/font/font.h>
+#include <librnd/font2/font.h>
 #include <librnd/hid/hid_inlines.h>
 #include <librnd/core/rotate.h>
 #include <librnd/core/safe_fs_dir.h>
@@ -113,7 +113,7 @@ static lht_node_t *hash_get(lht_node_t *hash, const char *name)
 #define HASH_GET(hash, name)        hash_get(hash, name)
 #define RND_LHT_ERROR               iolht_error
 
-#include <librnd/font/font_lht.h>
+#include <librnd/font2/font_lht.h>
 
 
 /* text object fields:
@@ -183,6 +183,7 @@ static rnd_font_t *rf_font_lookup_(rnd_design_t *hidlib, const char *name, const
 
 		if (rnd_font_load(hidlib, f, best->key, 0) != 0) {
 			/* failed to load the font, remove from hash, try another */
+			free(f);
 			htsp_popentry(&rf_map, best->key);
 			if (sch_rnd_font_score_debug)
 				fprintf(stderr, "  FAILED\n");
@@ -356,6 +357,7 @@ static void rf_text_update(csch_sheet_t *sheet, csch_text_t *t, csch_cpen_t *pen
 
 	/* get initial size */
 	rnd_font_string_bbox(cx, cy, f, text_str, 0, 0, 1, 1, 0, 0, 0, 0);
+	cx[0] = 0; /* undo the effect of left side whitespace: we started to render from 0;0 */
 	w = P2C(cx[1] - cx[0]);
 	h = P2C(f->height);
 
@@ -434,7 +436,7 @@ void sch_rnd_render_text_string(rnd_hid_gc_t gc, void *font, rnd_coord_t x, rnd_
 	ctx.gc = gc;
 
 	rnd_font_draw_string_justify(font, text_str, x, y, size, size, 0,
-		RND_FONT_MIRROR_Y, -3, 0, 0, RND_FONT_TINY_CHEAP,
+		RND_FONT_MIRROR_Y, -3, 0, RND_FONT_TINY_CHEAP,
 		0, 0, rf_draw_atom_cb, &ctx);
 }
 
@@ -445,7 +447,7 @@ void sch_rnd_render_text_string_scrotmir(rnd_hid_gc_t gc, void *font, rnd_coord_
 	ctx.gc = gc;
 
 	rnd_font_draw_string_justify(font, text_str, x, y, size, size, rot,
-		mirrory ? RND_FONT_MIRROR_Y : 0, -3, 0, 0, RND_FONT_TINY_CHEAP,
+		mirrory ? RND_FONT_MIRROR_Y : 0, -3, 0, RND_FONT_TINY_CHEAP,
 		0, 0, rf_draw_atom_cb, &ctx);
 }
 
@@ -495,7 +497,8 @@ static void rf_text_render(csch_sheet_t *sheet, rnd_hid_gc_t gc, csch_text_t *t,
 	rnd_rotate(&x, &y, xo, yo, cos(rad), sin(rad));
 
 	rnd_font_draw_string_justify(font, text_str, x, y, t->scale, t->scale, deg,
-		RND_FONT_MIRROR_Y, 0, 0, 0, RND_FONT_TINY_CHEAP,
+		RND_FONT_MIRROR_Y | RND_FONT_HTAB | RND_FONT_INVIS_TAB,
+		0, 0, RND_FONT_TINY_CHEAP,
 		C2P(t->extra_glyph), C2P(t->extra_spc), rf_draw_atom_cb, &ctx);
 }
 
@@ -553,8 +556,11 @@ static void sch_rnd_font_rnd_uninit(void)
 	htsp_entry_t *e;
 	for(e = htsp_first(&rf_map); e != NULL; e = htsp_next(&rf_map, e)) {
 		rnd_font_t *f = e->value;
-		if (f != &loadme)
+		if (f != &loadme) {
 			rnd_font_free(f);
+			free(f);
+		}
+		free(e->key);
 	}
 	htsp_uninit(&rf_map);
 	free(embf);
