@@ -1,7 +1,7 @@
 /*
  *                            COPYRIGHT
  *
- *  cschem - modular/flexible schematics editor - sch-rnd (executable)
+ *  sch-rnd - modular/flexible schematics editor - sch-rnd (executable)
  *  Copyright (C) 2019..2022 Tibor 'Igor2' Palinkas
  *
  *  (Supported by NLnet NGI0 PET Fund in 2022)
@@ -39,6 +39,7 @@
 #include <librnd/core/unit.h>
 #include <librnd/hid/hid_init.h>
 #include <librnd/hid/hid.h>
+#include <librnd/hid/tool.h>
 #include <librnd/core/conf.h>
 #include <librnd/core/hidlib.h>
 #include <librnd/hid/buildin.hidlib.h>
@@ -58,6 +59,7 @@
 #include <libcschem/util_export.h>
 #include <libcschem/event.h>
 #include <libcschem/csch_printf.h>
+#include <libcschem/hierarchy.h>
 
 #include "conf_core.h"
 #include "crosshair.h"
@@ -89,7 +91,6 @@ static rnd_conf_ignore_t sch_rnd_conf_ignores[] = {
 	{NULL, 0, 0, NULL}
 };
 
-#ifdef LIBRND41
 static int sch_rnd_human_coord(rnd_coord_t coord, double *out_val, const char **out_suffix)
 {
 	*out_val = P2C(coord);
@@ -102,8 +103,6 @@ static int sch_rnd_human_coord(rnd_coord_t coord, double *out_val, const char **
 
 	return 1;
 }
-#endif
-
 
 static char *main_path_init(char *exec_prefix)
 {
@@ -138,10 +137,7 @@ static char *main_path_init(char *exec_prefix)
 	rnd_app.conf_userdir_path = CONF_USER_DIR;
 	rnd_app.conf_user_path = rnd_concat(CONF_USER_DIR, "/conf_core.lht", NULL);
 	rnd_app.conf_ignores = sch_rnd_conf_ignores;
-
-#ifdef LIBRND41
 	rnd_app.human_coord = sch_rnd_human_coord;
-#endif
 
 	return exec_prefix;
 }
@@ -237,6 +233,7 @@ static void gui_act_init(void)
 
 extern void sch_rnd_main_act_init2(void);
 extern void sch_rnd_file_act_init2(void);
+extern void sch_rnd_font_act_init2(void);
 extern void sch_rnd_select_act_init2(void);
 extern void sch_rnd_draw_init2(void);
 extern void sch_rnd_buffer_init2(void);
@@ -260,6 +257,7 @@ static void sch_rnd_main_init(void)
 	sch_rnd_main_act_init2();
 	sch_rnd_sheet_init2();
 	sch_rnd_file_act_init2();
+	sch_rnd_font_act_init2();
 	sch_rnd_select_act_init2();
 	sch_rnd_font_init2();
 	sch_rnd_draw_init2();
@@ -267,12 +265,22 @@ static void sch_rnd_main_init(void)
 /*	rnd_main_act_init2();*/
 }
 
+#define LIBCSCHEM_CFG_BIND(csch_cfg, native) \
+	do { \
+		if (sizeof(*csch_cfg) == sizeof(native)) \
+			csch_cfg = &native; \
+		else \
+			rnd_message(RND_MSG_ERROR, "Can not enable " #native " on this arch. Please report this bug.\n"); \
+	} while(0)
+
+/* Bind conf_core natives to libcschem config API; this is needed because
+   libcschem can't access conf_core directly as it can't depend on app code */
 static void libcschem_cfg_init(void)
 {
-	if (sizeof(*csch_cfg_multiport_net_merge) == sizeof(conf_core.compile.multiport_net_merge))
-		csch_cfg_multiport_net_merge = &conf_core.compile.multiport_net_merge;
-	else
-		rnd_message(RND_MSG_ERROR, "Can not enable conf_core.compile.multiport_net_merge on this arch. Please report this bug.\n");
+	LIBCSCHEM_CFG_BIND(csch_cfg_multiport_net_merge, conf_core.compile.multiport_net_merge);
+	LIBCSCHEM_CFG_BIND(csch_cfg_hier_net_allow_rename, conf_core.compile.hierarchic.net_allow_rename);
+	LIBCSCHEM_CFG_BIND(csch_cfg_hier_net_rename_prefer_top, conf_core.compile.hierarchic.net_rename_prefer_top);
+	csch_app_load_sheet_cb = sch_rnd_app_load_sheet;
 }
 
 extern void sch_rnd_draw_uninit(void);
@@ -282,6 +290,8 @@ extern void sch_rnd_project_uninit(void);
 
 void sch_rnd_main_uninit(void)
 {
+	TODO("librnd4.3: can't enable this because librnd sefaults on exit when it prints a log message and the gui is already uninited; fixed inlibrnd 4.3");
+/*	rnd_tool_uninit();*/
 	sch_rnd_multi_uninit();
 	rnd_funchash_uninit();
 	csch_plug_io_act_uninit();
@@ -296,6 +306,8 @@ void sch_rnd_main_uninit(void)
 	main_path_uninit();
 	fgw_atexit();
 	conf_core_uninit();
+	rnd_log_uninit();
+	csch_uninit_last();
 }
 
 extern const char *rnd_conf_internal;
@@ -386,7 +398,7 @@ TODO("action list");
 	for(n = 0; ga.hid_argc > 0; n++, ga.hid_argc--) {
 		const char *fn = ga.hid_argv[n];
 		csch_sheet_t *sheet;
-		int is_project;
+		int is_project = 0;
 
 		sheet = sch_rnd_multi_load(fn, fmt, &is_project);
 		if ((first_fn == NULL) && !is_project)
@@ -435,6 +447,7 @@ TODO("action list");
 		sch_rnd_multi_switch_to_initial(first_sheet);
 
 	if (rnd_main_exported(&ga, &first_sheet->hidlib, 0, &appspec)) {
+		rnd_log_print_uninit_errs("Export finished");
 		sch_rnd_sheet_free_all();
 		sch_rnd_main_uninit();
 		rnd_main_args_uninit(&ga);
